#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging


LOG = logging.getLogger(__name__)


def demo_handler(view_fn):
  def decorator(request, *args, **kwargs):
    try:
      return view_fn(request, *args, **kwargs)
    except Exception, e:
      from search.api import SolrApi
      if '/solr/twitter_demo/select' in str(e):
        return SolrApi._get_json(TWITTER_SEARCH_RESPONSE)
      elif '/solr/yelp_demo/select' in str(e):
        return SolrApi._get_json(YELP_SEARCH_RESPONSE)
      elif '/solr/log_demo/select' in str(e):
        return SolrApi._get_json(LOG_SEARCH_RESPONSE)
      else:
        raise e
  return decorator


# Adding "is_demo" property to each request

TWITTER_SEARCH_RESPONSE = """
  {"responseHeader":{"is_demo":true,"status":0,"QTime":1522,"params":{"facet":"true","facet.mincount":"1","f.user_followers_count.facet.range.gap":"100","f.user_statuses_count.facet.range.gap":"1000","facet.limit":"100","facet.range":["created_at","user_followers_count","user_statuses_count"],"hl.fl":"text","f.user_followers_count.facet.range.end":"1000","wt":"json","hl":"true","rows":"15","f.created_at.facet.range.start":"NOW-400DAYS/DAYS","f.created_at.facet.limit":"-1","f.user_statuses_count.facet.range.end":"10000","f.user_followers_count.facet.range.start":"0","f.created_at.facet.range.end":"NOW","user.name":"hue","start":"0","facet.sort":"count","q":"love","doAs":"romain","f.created_at.facet.range.gap":"+15MINUTES","f.user_statuses_count.facet.range.start":"0","facet.field":"user_location"}},"response":{"numFound":777,"start":0,"docs":[{"text":"@Vivibrot jogja with love love love love :p","user_statuses_count":4982,"user_screen_name":"atyhasan","user_location":"Indonesia","user_followers_count":186,"id":"438657307677712384","user_name":"Hidayati Hasan","source":"<a href='http://twitter.com/download/iphone' rel='nofollow'>Twitter for iPhone</a>","in_reply_to_status_id":[438656994791018496],"retweet_count":0,"in_reply_to_user_id":248240357,"created_at":"2014-02-25T20:50:26Z","user_friends_count":148,"_version_":1461150195223887877},{"text":"Learn to love at love to learn :))","user_statuses_count":8980,"user_screen_name":"Cocacolasii","user_location":"Philippines","user_followers_count":344,"id":"438650705805451264","user_name":"G E L ☆","source":"<a href='http://blackberry.com/twitter' rel='nofollow'>Twitter for BlackBerry®</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T20:24:12Z","user_friends_count":328,"_version_":1461150195034095622},{"text":"@Northam_Scott love it","user_statuses_count":2194,"user_screen_name":"senorsanta","user_location":"London","user_followers_count":1321,"id":"438622478139932672","user_name":"⭐João Da Mata⭐","source":"<a href='http://twitter.com/#!/download/ipad' rel='nofollow'>Twitter for iPad</a>","in_reply_to_status_id":[438621604789387264],"retweet_count":0,"in_reply_to_user_id":1583797404,"created_at":"2014-02-25T18:32:02Z","user_friends_count":74,"_version_":1461150192060334083},{"text":"For the Love #TheLegalWifeDay23","user_statuses_count":6680,"user_screen_name":"shammanekay","user_location":"philippines","user_followers_count":104,"id":"438646587019902976","user_name":"shammane Zaldivia","source":"<a href='https://mobile.twitter.com' rel='nofollow'>Mobile Web (M2)</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T20:07:50Z","user_friends_count":429,"_version_":1461150192210280454},{"text":"#CarmelaDay23 LOVE!","user_statuses_count":387,"user_screen_name":"WeLoveGMA33","user_location":"Philippines","user_followers_count":2,"id":"438659010560933888","user_name":"We Love GMA","source":"<a href='https://about.twitter.com/products/tweetdeck' rel='nofollow'>TweetDeck</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T20:57:12Z","user_friends_count":11,"_version_":1461150192468230150},{"text":"▶ Love Words","user_statuses_count":16941,"user_screen_name":"fdhanh","user_location":"Japan","user_followers_count":170,"id":"438661472596410368","user_name":"Miku Hatsune♪","source":"<a href='http://twittbot.net/' rel='nofollow'>twittbot.net</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T21:06:59Z","user_friends_count":57,"_version_":1461150192477667337},{"text":"♪Lost Love♪","user_statuses_count":2456,"user_screen_name":"alain_lionel","user_location":"Argentina","user_followers_count":138,"id":"438827181159219200","user_name":"▲Mirage▲","source":"web","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-26T08:05:27Z","user_friends_count":173,"_version_":1461150194458427395},{"text":"♫ So this is love  ♫","user_statuses_count":3552,"user_screen_name":"mrcrptrc","user_location":"Philippines","user_followers_count":426,"id":"438651418837131264","user_name":"Maricar Patricia ⭐️","source":"web","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T20:27:02Z","user_friends_count":371,"_version_":1461150195041435648},{"text":"#CarmelaDay23 LOVE!","user_statuses_count":377,"user_screen_name":"WeLoveGMA34","user_location":"Philippines","user_followers_count":1,"id":"438659010552537088","user_name":"We Love GMA","source":"<a href='https://about.twitter.com/products/tweetdeck' rel='nofollow'>TweetDeck</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T20:57:12Z","user_friends_count":12,"_version_":1461150195416825867},{"text":"LOVE LOVE LOVE this photo shoot! '@AlexandraHind #VOGUE http://t.co/23XnP233au '","user_statuses_count":5812,"user_screen_name":"chelleajohnson","user_location":"London","user_followers_count":1292,"id":"438629985969262592","user_name":"Michelle Johnson","source":"web","in_reply_to_status_id":[438306632037335040],"retweet_count":0,"in_reply_to_user_id":177920554,"created_at":"2014-02-25T19:01:52Z","user_friends_count":1952,"_version_":1461150192135831560},{"text":"@pevikachuu LOVE IS FEELING~ LOVE IS MORE PAIN~","user_statuses_count":28769,"user_screen_name":"bibillopato_","user_location":"Jakarta","user_followers_count":842,"id":"438606623678808064","user_name":"Nabila Makha Suci P","source":"<a href='http://twitter.com/download/iphone' rel='nofollow'>Twitter for iPhone</a>","in_reply_to_status_id":[438603394500935680],"retweet_count":0,"in_reply_to_user_id":85562866,"created_at":"2014-02-25T17:29:02Z","user_friends_count":625,"_version_":1461150194684919818},{"text":"Our love is not in line:')","user_statuses_count":2215,"user_screen_name":"MylaFZN","user_location":"Indonesia","user_followers_count":245,"id":"438588143571173376","user_name":"MilaCabiita","source":"<a href='http://blackberry.com/twitter' rel='nofollow'>Twitter for BlackBerry®</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T16:15:36Z","user_friends_count":99,"_version_":1461150191792947201},{"text":"I will love you unconditionally :)","user_statuses_count":4068,"user_screen_name":"itsMeNicole006","user_location":"Philippines","user_followers_count":168,"id":"438610201428520960","user_name":"Milk :)","source":"<a href='https://twitter.com/download/android' rel='nofollow'>Twitter for  Android</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T17:43:15Z","user_friends_count":385,"_version_":1461150191952330761},{"text":"i love food","user_statuses_count":7706,"user_screen_name":"myraju_","user_location":"Thailand","user_followers_count":361,"id":"438630489293733888","user_name":"Ming","source":"<a href='http://twitter.com/download/iphone' rel='nofollow'>Twitter for iPhone</a>","in_reply_to_status_id":[-1],"retweet_count":0,"in_reply_to_user_id":-1,"created_at":"2014-02-25T19:03:52Z","user_friends_count":257,"_version_":1461150192141074436},{"text":"@TheCurlyHarrry_ i love you to🎈🎅","user_statuses_count":24472,"user_screen_name":"_xmxx_","user_location":"London","user_followers_count":1183,"id":"438639175685140480","user_name":"♕مارلي♕","source":"<a href='http://twitter.com/download/iphone' rel='nofollow'>Twitter for iPhone</a>","in_reply_to_status_id":[438638914216427520],"retweet_count":0,"in_reply_to_user_id":558051355,"created_at":"2014-02-25T19:38:23Z","user_friends_count":400,"_version_":1461150192349741063}]},"facet_counts":{"facet_queries":{},"facet_fields":{"user_location":["london",135,"philippines",107,"argentina",90,"indonesia",80,"uk",71,"usa",43,"france",40,"new york",37,"malaysia",29,"brasil",27,"paris",22,"venezuela",21,"thailand",16,"españa",15,"japan",13,"jakarta",11,"istanbul",9,"madrid",8,"türkiye",1,"大阪",1,"東京",1]},"facet_dates":{},"facet_ranges":{"created_at":{"counts":["2014-02-25T16:00:00Z",8,"2014-02-25T16:15:00Z",14,"2014-02-25T16:30:00Z",5,"2014-02-25T16:45:00Z",6,"2014-02-25T17:00:00Z",12,"2014-02-25T17:15:00Z",13,"2014-02-25T17:30:00Z",8,"2014-02-25T17:45:00Z",8,"2014-02-25T18:00:00Z",9,"2014-02-25T18:15:00Z",12,"2014-02-25T18:30:00Z",12,"2014-02-25T18:45:00Z",6,"2014-02-25T19:00:00Z",18,"2014-02-25T19:15:00Z",16,"2014-02-25T19:30:00Z",13,"2014-02-25T19:45:00Z",12,"2014-02-25T20:00:00Z",17,"2014-02-25T20:15:00Z",10,"2014-02-25T20:30:00Z",17,"2014-02-25T20:45:00Z",14,"2014-02-25T21:00:00Z",19,"2014-02-25T21:15:00Z",15,"2014-02-25T21:30:00Z",13,"2014-02-25T21:45:00Z",13,"2014-02-25T22:00:00Z",17,"2014-02-25T22:15:00Z",14,"2014-02-25T22:30:00Z",17,"2014-02-25T22:45:00Z",15,"2014-02-25T23:00:00Z",9,"2014-02-25T23:15:00Z",22,"2014-02-25T23:30:00Z",10,"2014-02-25T23:45:00Z",12,"2014-02-26T00:00:00Z",12,"2014-02-26T00:15:00Z",14,"2014-02-26T00:30:00Z",23,"2014-02-26T00:45:00Z",12,"2014-02-26T01:00:00Z",15,"2014-02-26T01:15:00Z",13,"2014-02-26T01:30:00Z",7,"2014-02-26T01:45:00Z",20,"2014-02-26T02:00:00Z",15,"2014-02-26T02:15:00Z",21,"2014-02-26T02:30:00Z",9,"2014-02-26T02:45:00Z",11,"2014-02-26T03:00:00Z",8,"2014-02-26T03:15:00Z",12,"2014-02-26T03:30:00Z",7,"2014-02-26T03:45:00Z",15,"2014-02-26T04:00:00Z",2,"2014-02-26T04:15:00Z",2,"2014-02-26T04:30:00Z",5,"2014-02-26T04:45:00Z",4,"2014-02-26T05:00:00Z",12,"2014-02-26T05:15:00Z",8,"2014-02-26T05:30:00Z",8,"2014-02-26T05:45:00Z",9,"2014-02-26T06:00:00Z",14,"2014-02-26T06:15:00Z",12,"2014-02-26T06:30:00Z",8,"2014-02-26T06:45:00Z",20,"2014-02-26T07:00:00Z",8,"2014-02-26T07:15:00Z",10,"2014-02-26T07:30:00Z",6,"2014-02-26T07:45:00Z",6,"2014-02-26T08:00:00Z",16,"2014-02-26T08:15:00Z",6,"2014-02-26T08:30:00Z",1],"gap":"+15MINUTES","start":"2013-01-23T00:00:00Z","end":"2014-02-27T00:45:00Z"},"user_followers_count":{"counts":["0",147,"100",111,"200",80,"300",74,"400",48,"500",33,"600",19,"700",30,"800",20,"900",11],"gap":100,"start":0,"end":1000},"user_statuses_count":{"counts":["0",140,"1000",71,"2000",69,"3000",45,"4000",40,"5000",30,"6000",36,"7000",28,"8000",23,"9000",19],"gap":1000,"start":0,"end":10000}}},"highlighting":{"438657307677712384":{"text":["@Vivibrot jogja with <em>love</em> <em>love</em> <em>love</em> <em>love</em> :p"]},"438650705805451264":{"text":["Learn to <em>love</em> at <em>love</em> to learn :))"]},"438622478139932672":{"text":["@Northam_Scott <em>love</em> it"]},"438646587019902976":{"text":["For the <em>Love</em> #TheLegalWifeDay23"]},"438659010560933888":{"text":["#CarmelaDay23 <em>LOVE</em>!"]},"438661472596410368":{"text":["▶ <em>Love</em> Words"]},"438827181159219200":{"text":["♪Lost <em>Love</em>♪"]},"438651418837131264":{"text":["♫ So this is <em>love</em>  ♫"]},"438659010552537088":{"text":["#CarmelaDay23 <em>LOVE</em>!"]},"438629985969262592":{"text":["<em>LOVE</em> <em>LOVE</em> <em>LOVE</em> this photo shoot! '@AlexandraHind #VOGUE http://t.co/23XnP233au '"]},"438606623678808064":{"text":["@pevikachuu <em>LOVE</em> IS FEELING~ <em>LOVE</em> IS MORE PAIN~"]},"438588143571173376":{"text":["Our <em>love</em> is not in line:')"]},"438610201428520960":{"text":["I will <em>love</em> you unconditionally :)"]},"438630489293733888":{"text":["i <em>love</em> food"]},"438639175685140480":{"text":["@TheCurlyHarrry_ i <em>love</em> you to🎈🎅"]}}}
  """

YELP_SEARCH_RESPONSE = """
  {"responseHeader":{"is_demo":true,"status":0,"QTime":5,"params":{"f.stars.facet.range.gap":"1","facet":"true","sort":"stars desc","facet.mincount":"1","facet.limit":"100","facet.range":["stars","review_count"],"f.review_count.facet.range.gap":"10","hl.fl":"text","f.review_count.facet.range.start":"0","wt":"json","hl":"true","rows":"15","f.stars.facet.range.end":"6","f.review_count.facet.range.end":"100","user.name":"hue","start":"0","facet.sort":"count","q":"good","f.stars.facet.range.start":"0","doAs":"romain","facet.field":"open"}},"response":{"numFound":408,"start":0,"docs":[{"business_id":"rxQ2PIjhAx6dgAqUalf99Q","cool":0,"date":"2012-09-09","funny":0,"id":"-v-shjbxoj7hpU62yn6vag","stars":5,"text":"Never having dealt with a Discount Tire in Phoenix before (only in Texas, and their service has been great all the time), I came in not knowing what kind of service to expect. I was pleasantly surprised! The store had a good amount of parking, and the store front was clean, open and had adequate seating around the perimeters. There were already 5-6 people sitting and waiting for their cars to be finished, and 2 people waiting infront of me to see a representative. All the guys working there were busy helping someone, yet were saying hello to us and reassuring they'd get to us very soon. I didn't wait an unneccesary amount of time and then was helped by Wally. He inspected my tires, recommended a plan of action gave me options, and we headed back into the store. After I made my decision to replace 2 of the tires, he checked their stock, and unfortunately they didn't have them there. BUT he called another store and had them picked up for the next morning, so I could just come back the next day to this same store and get those bad boys installed. Great! The prices were not as heafty (relief), and although there were a good amount of people the next day waiting for their cars to also be serviced, it took exactly the amount of time they'd told me (45 min). They apologized for the wait, but no need for that! The guy who helped me on the 2nd day (I'm sorry I forgot his name) was also very professional and great. I recommend this store and will be bringing my car here in the future most definitely. Thanks, guys!TIP: If you know you need to bring your car in, you can make an appointment with them online. They take walk-ins, but I believe it helps them manage their customers easier, and that in turn benefits you.","type":"business","useful":0,"user_id":"HLbhD2OyiMCUDRR4c1iXaw","name":"Discount Tire","full_address":"7920 W Bell Road Glendale, AZ 85308","latitude":33.63867,"longitude":-112.22953,"neighborhoods":"[]","open":"True","review_count":11,"state":"AZ","_version_":1461150196153974786},{"business_id":"1udzLbeQxmLTbOtgyVP0kQ","cool":0,"date":"2012-11-17","funny":0,"id":"YdrffsqmsOZX81hYvK8Qwg","stars":5,"text":"Delish & authentic! The serving was generous (meat combo fed 2). The injera was good (not too thick/doughy). Spicy and yummy. I would like to return and experience their coffee ceremony.","type":"business","useful":2,"user_id":"TmC0qdyUgXCkKK47vwLRPQ","name":"Gojo Ethiopian Restauran","full_address":"3015 E Thomas Rd Ste 101 Phoenix, AZ 85016","latitude":33.479607,"longitude":-112.01648,"neighborhoods":"[]","open":"True","review_count":8,"state":"AZ","_version_":1461150196260929536},{"business_id":"UuAxFxxabj60OrMFdurShw","cool":0,"date":"2012-12-05","funny":0,"id":"915mO7l6Lmz_vWqfnIgwkg","stars":5,"text":"Everyone here is very helpful and friendly.  No matter who I come in contact with I can tell they love animals and care about me and my pet.  The vet calls me himself to discuss my cat's test results.  They also made sure to give me a lot of information regarding the medications they are putting her on.  I feel that my family gets good care from this vet.  We ended up moving about 15 minutes away, but still make the trip to see this vet.","type":"business","useful":0,"user_id":"6gt6iJB1hiFvSrms5RWxgA","name":"KC Animal Hospital","full_address":"10607 N Frank L Wright Ste 201 Scottsdale, AZ 85259","latitude":33.582962,"longitude":-111.83213,"neighborhoods":"[]","open":"True","review_count":5,"state":"AZ","_version_":1461150196275609605},{"business_id":"jVaujSVCpH9c2qb7XkvpnA","cool":0,"date":"2011-03-22","funny":0,"id":"DrFU7tPfdY-95Pm47r72nA","stars":5,"text":"I am so not a fan of the dentist.  But Dr. Ito and his staff changed that.  Professional, yet relaxed attitudes.  From the moment you book the appointment to the time you leave you feel like are well cared for and that you aren't just a number or a dollar sign.While in the waiting area I flipped through Dr. Ito's 'Smile' book that shows some of his work.  WOW!  He makes teeth look real, not like Chiclets.  He does some fabulous cosmetic work!  I went in to have a crown put on and it was quick and painless. So quick that I didn't get to finish watching the movie!   Each chair has headphones and a flat screen that shows a movie (he keeps a good stock of the latest movies.).  Who'd have thought that going to the dentist would keep me so entertained?I've had to bring my son to each appointment as I haven't had anyone to keep him.  Does that matter? Nope.  Dr. Ito's office is equipped with a children's area in the back where the kids can watch a movie, read a book or play a game.  He thinks of everything.I have been recommending him to everyone I meet.  I can't say enough good things about Dr. Ito and his staff.  You won't regret going to the dentist if Sonoran Hills Dental is your new dental office.","type":"business","useful":0,"user_id":"n1ZKHhVozfzCDLEgsZTrkQ","name":"Sonoran Hills Dental","full_address":"4909 E Chandler Blvd Ste 501 Phoenix, AZ 85048","latitude":33.30414,"longitude":-111.97839,"neighborhoods":"[]","open":"True","review_count":5,"state":"AZ","_version_":1461150196280852480},{"business_id":"tenKOmTRi2rjZAWwNCDv6w","cool":1,"date":"2012-09-22","funny":1,"id":"0UBnLhVxAfBI8e36wTupXA","stars":5,"text":"This is the only auto repair place I've ever seen with 5 stars solid, and they totally deserve it.My boyfriend has a 1969 Pontiac Firebird that hasn't run in several years. After sinking lots of his own time into tinkering with it and $1000 to the awful Meineke down the street, he gave up... until my car's transmission went for good, and he decided to fix the Bird (yeah!) so I could drive it (yes, I am super lucky!)Whitey's diagnosed it as the carburetor dumping fuel (which is what Meineke charged $500 to replace.. hmmm...). Upon hearing the story of Meineke, Whitey's told him to order the part himself to avoid the markup and they'd put it in for just labor.  Awesome!We took it home, but it wasn't running quite right still, and then failed emissions. Back to the shop it went.Whitey's took it back, spent ALL WEEKEND on it doing various minor things. At one point, they called and said 'We've done this, this and this, and it's still not quite right, so we're gonna keep it another day.'   I was glad they were admitting they actually felt it to be not right and were determined to fix it, versus just handing it back again... but I was dreading the repair bill.We picked it up Tuesday. The bill? $7. That's right - SEVEN DOLLARS. They only charged him for a spark plug and oil filter. That is amazing to me - I was expecting several hundred dollars for their time.  The car runs beautifully now, and they were extremely pleasant the entire time. I'm taking my alleged defunct car to them for a second opinion next week. I HIGHLY recommend Whitey's!","type":"business","useful":2,"user_id":"ysINNe8YpbOW_-5T5CSyhw","name":"Whitey's Auto Repair","full_address":"3014 N Civic Center Plz Scottsdale, AZ 85251","latitude":33.482227,"longitude":-111.92211,"neighborhoods":"[]","open":"True","review_count":23,"state":"AZ","_version_":1461150196286095364},{"business_id":"J-c25QK8xBqv1P-huUkoTQ","cool":0,"date":"2010-08-27","funny":0,"id":"nAJ0HvYhoUjo5wKMewK7iA","stars":5,"text":"I LOVE Laser for Less (Chandler)!  If you are going anywhere else for Laser hair removal you need to stop now and call to make an appointment with Laser for Less, Chandler.   I wasted a lot of $$$ at another Laser removal place before I found Laser for Less and now I am kicking myself for not going to them first.  If you pay to wax any part of your body you should be lasering that hair off.  It works better, lasts longer, and in the long run is cheaper.  Pros - 1. Free touch ups 14-18 days after your first appointment if they missed 1 hair they will touch up for free.  Never had a problem with them discouraging me, in fact it was the opposite they give me reminder cards so I know when my touch ups are due.  They encourage you to come in.2. They can also charge you by the laser shot instead of a whole area.  So if you only have a few random hairs they will just charge 10 cent per shot on the legs or 70 cents on the face or smaller areas.  You can't beat that deal.3. I am pretty sure they beat or match all prices in town.4. Once they laser the hair, all your hair will fall out and wont grow back for like 2 months, its better than waxing.  Not all laser removal places use the type of laser that does this place does.  That is why I recommend them, it works a lot better!5. They actually care about getting rid of unwanted hair and will go over all their policies with you.  Kristen is who I go to and she is great.  I have been to almost all their girls and they are good. 6. In fact when it was my birthday month I didn't know I got some discount and they back credited me for it, I feel like they are always trying to get me the best deal.  7. They always have deals.  Just last month they had their underarm extravaganza where you can get your underarms done for $30!!! That is so cheap.  They let you buy up to 3 sessions.  I referred a friend and she loved it so much because she didn't have to shave her underarms the WHOLE summer.  8. I love Laser for Less, CHANDLER!!!Cons - never having to shave again, darn.","type":"business","useful":3,"user_id":"mqueHx5CFC9IsZxLQb3H9g","name":"Laser For Less","full_address":"2855 W Ray Rd Ste 7 Chandler, AZ 85224","latitude":33.32031,"longitude":-111.89044,"neighborhoods":"[]","open":"True","review_count":9,"state":"AZ","_version_":1461150196296581123},{"business_id":"6sELDdbWi_Cjwp5uZ8PmGw","cool":8,"date":"2010-06-21","funny":10,"id":"ZucxvCvKaw7ViaKU00HYGg","stars":5,"text":"Ice Ice Baby...Ice Ice Baby...All right stop collaborate and listen;Ice Now is here and it's got my attention Make some drinks - not warm but frosty;And believe me when I say they are not costlyWill it ever melt? yo I don't know.Give them a call and they'll showTo the extreme I yelp like a sandal;Ice your drinks and get lit like a candle... Ice ice babyPS - not sure where the sandal line came in but writing a review in lyric form is a challenge!  This is the best I can do for now...Need ice for an event?  Call Ice Now! They offer free delivery and their pricing is surprisingly cheap.And good luck getting the Ice Ice Baby lyrics out of your head... (sorry)!","type":"business","useful":7,"user_id":"fczQCSmaWF78toLEmb0Zsw","name":"Ice Now","full_address":"505 S Country Club Dr Mesa, AZ 85210","latitude":33.40589,"longitude":-111.83969,"neighborhoods":"[]","open":"True","review_count":4,"state":"AZ","_version_":1461150196312309760},{"business_id":"621ZMZH68DilrRgBRw1R6A","cool":0,"date":"2011-12-08","funny":0,"id":"M1LxHqmnzohCNyLamfbmow","stars":5,"text":"I love Dr. Scott!! He is the funniest doctor I've ever met and he makes me feel good too! Dr. Scott is always willing to bend over backwards to make sure you are provided the best service and treatment possible. The massage therapists rock too! I will be a lifelong patient of Integrated Chiropractic.","type":"business","useful":0,"user_id":"PoLTut88ZmgFsnFvsTy5XQ","name":"Integrated Chiropractic","full_address":"14850 N Scottsdale Rd Ste 375 Scottsdale, AZ 85254","latitude":33.622093,"longitude":-111.92598,"neighborhoods":"[]","open":"True","review_count":3,"state":"AZ","_version_":1461150196312309764},{"business_id":"ZRJwVLyzEJq1VAihDhYiow","cool":0,"date":"2011-07-27","funny":0,"id":"IjZ33sJrzXqU-0X6U8NwyA","stars":4,"text":"I have no idea why some people give bad reviews about this place. It goes to show you, you can please everyone. They are probably griping about something that their own fault...there are many people like that.In any case, my friend and I arrived at about 5:50 PM this past Sunday. It was pretty crowded, more than I thought for a Sunday evening and thought we would have to wait forever to get a seat but they said we'll be seated when the girl comes back from seating someone else. We were seated at 5:52 and the waiter came and got our drink orders. Everyone was very pleasant from the host that seated us to the waiter to the server. The prices were very good as well. We placed our orders once we decided what we wanted at 6:02. We shared the baked spaghetti calzone and the small 'Here's The Beef' pizza so we can both try them. The calzone was huge and we got the smallest one (personal) and got the small 11' pizza. Both were awesome! My friend liked the pizza better and I liked the calzone better. The calzone does have a sweetish sauce but that's how I like my sauce!We had to box part of the pizza to take it home and we were out the door by 6:42. So, everything was great and not like these bad reviewers. That goes to show you that  you have to try these things yourself because all these bad reviewers have some serious issues.","type":"business","useful":0,"user_id":"0a2KyEL0d3Yb1V6aivbIuQ","name":"Spinato's Pizzeria","full_address":"4848 E Chandler Blvd Phoenix, AZ 85044","latitude":33.305607,"longitude":-111.97876,"neighborhoods":"[]","open":"True","review_count":102,"state":"AZ","_version_":1461150196147683328},{"business_id":"6oRAC4uyJCsJl1X0WZpVSA","cool":0,"date":"2012-06-14","funny":0,"id":"IESLBzqUCLdSzSqm0eCSxQ","stars":4,"text":"love the gyro plate. Rice is so good and I also dig their candy selection :)","type":"business","useful":1,"user_id":"0hT2KtfLiobPvh6cDC8JQg","name":"Haji-Baba","full_address":"1513 E  Apache Blvd Tempe, AZ 85281","latitude":33.414345,"longitude":-111.91303,"neighborhoods":"[]","open":"True","review_count":265,"state":"AZ","_version_":1461150196148731904},{"business_id":"6ozycU1RpktNG2-1BroVtw","cool":0,"date":"2012-01-05","funny":0,"id":"1uJFq2r5QfJG_6ExMRCaGw","stars":4,"text":"General Manager Scott Petello is a good egg!!! Not to go into detail, but let me assure you if you have any issues (albeit rare) speak with Scott and treat the guy with some respect as you state your case and I'd be surprised if you don't walk out totally satisfied as I just did. Like I always say..... 'Mistakes are inevitable, it's how we recover from them that is important'!!!Thanks to Scott and his awesome staff. You've got a customer for life!! .......... :^)","type":"business","useful":0,"user_id":"vYmM4KTsC8ZfQBg-j5MWkw","name":"Discount Tire","full_address":"1357 S Power Road Mesa, AZ 85206","latitude":33.391026,"longitude":-111.68448,"neighborhoods":"[]","open":"True","review_count":5,"state":"AZ","_version_":1461150196148731906},{"business_id":"zp713qNhx8d9KCJJnrw1xA","cool":7,"date":"2010-02-12","funny":4,"id":"riFQ3vxNpP4rWLk_CSri2A","stars":4,"text":"Drop what you're doing and drive here. After I ate here I had to go back the next day for more.  The food is that good.This cute little green building may have gone competely unoticed if I hadn't been driving down Palm Rd to avoid construction.  While waiting to turn onto 16th Street the 'Grand Opening' sign caught my eye and my little yelping soul leaped for joy!  A new place to try!It looked desolate from the outside but when I opened the door I was put at easy by the decor, smell and cleanliness inside.  I ordered dinner for two, to go.  The menu was awesome.  I loved seeing all the variety: poblano peppers, mole, mahi mahi, mushrooms...something wrapped in banana leaves.  It made it difficult to choose something.  Here's what I've had so far: La Condesa Shrimp Burro and Baja Sur Dogfish Shark Taco.  They are both were very delicious meals but the shrimp burro stole the show.  So much flavor.  I snagged some bites from my hubbys mole and mahi mahi burros- mmmm such a delight.  The salsa bar is endless.  I really stocked up.  I was excited to try the strawberry salsa but it was too hot, in fact it all was, but I'm a big wimp when it comes to hot peppers. The horchata is handmade and delicious.  They throw pecans and some fruit in there too which is a yummy bonus!As if the good food wasn't enough to win me over the art in this restaurant sho did!  I'm a sucker for Mexican folk art and Frida Kahlo is my Oprah.  There's a painting of her and Diego hanging over the salsa bar, it's amazing.  All the paintings are great, love the artist.","type":"business","useful":7,"user_id":"wFweIWhv2fREZV_dYkz_1g","name":"La Condesa Gourmet Taco Shop","full_address":"1919 N 16th St Phoenix, AZ 85006","latitude":33.46913,"longitude":-112.04751,"neighborhoods":"[]","open":"True","review_count":307,"state":"AZ","_version_":1461150196148731908},{"business_id":"e9nN4XxjdHj4qtKCOPq_vg","cool":1,"date":"2011-10-21","funny":0,"id":"3rPt0LxF7rgmEUrznoH22w","stars":4,"text":"Wonderful Vietnamese sandwich shoppe. Their baguettes are great hot out of the oven with butter or in one of their many sandwich choices. They have a modest selection of baked goods along with some of the best egg rolls around. Bring cash or your ATM card as no credit cards are accepted but they have an ATM on premises.","type":"business","useful":1,"user_id":"C1rHp3dmepNea7XiouwB6Q","name":"Lee's Sandwiches","full_address":"1901 W Warner Rd Chandler, AZ 85224","latitude":33.334713,"longitude":-111.87479,"neighborhoods":"[]","open":"True","review_count":192,"state":"AZ","_version_":1461150196149780484},{"business_id":"h53YuCiIDfEFSJCQpk8v1g","cool":1,"date":"2010-01-11","funny":0,"id":"cGnKNX3I9rthE0-TH24-qA","stars":4,"text":"They have a limited time thing going on right now with BBQ chicken pizza (not sure how long it's going to last) but let me just say it was amazing.  Probably THE best BBQ Chicken pizza I have ever had.  I have tried other things too, like the tomato basil soup, and many of their sandwiches ... very good, very fresh - every time.  The 5 stars is for the pizza, but if I were to rate Jason's Deli over all they would get about a 4.","type":"business","useful":2,"user_id":"UPtysDF6cUDUxq2KY-6Dcg","name":"Jason's Deli","full_address":"1065 E Baseline Rd Gilbert, AZ 85233","latitude":33.37962,"longitude":-111.809425,"neighborhoods":"[]","open":"True","review_count":36,"state":"AZ","_version_":1461150196149780485},{"business_id":"WGNIYMeXPyoWav1APUq7jA","cool":1,"date":"2011-12-23","funny":0,"id":"FvEEw1_OsrYdvwLV5Hrliw","stars":4,"text":"Good tattoo shop. Clean space, multiple artists to choose from and books of their work are available for you to look though and decide who's style most mirrors what you're looking for. I chose Jet to do a cover-up for me and he worked with me on the design and our ideas and communication flowed very well. He's a very personable guy, is friendly and keeps the conversation going while he's working on you, and he doesn't dick around (read: He starts to work and continues until the job is done). He's very professional and informative. Good customer service combines with talent at the craft.","type":"business","useful":2,"user_id":"Xm8HXE1JHqscXe5BKf0GFQ","name":"The Lady Luck Tattoo Gallery","full_address":"961 E Guadalupe Rd Tempe, AZ 85283","latitude":33.36376,"longitude":-111.9273,"neighborhoods":"[]","open":"True","review_count":25,"state":"AZ","_version_":1461150196149780486}]},"facet_counts":{"facet_queries":{},"facet_fields":{"open":["true",360,"fals",48]},"facet_dates":{},"facet_ranges":{"stars":{"counts":["1",1,"2",12,"3",159,"4",228,"5",8],"gap":1,"start":0,"end":6},"review_count":{"counts":["0",38,"10",31,"20",28,"30",34,"40",22,"50",20,"60",37,"70",19,"80",14,"90",14],"gap":10,"start":0,"end":100}}},"highlighting":{"-v-shjbxoj7hpU62yn6vag":{"text":[" surprised! The store had a <em>good</em> amount of parking, and the store front was clean, open and had adequate"]},"YdrffsqmsOZX81hYvK8Qwg":{"text":["Delish & authentic! The serving was generous (meat combo fed 2). The injera was <em>good</em> (not too thick"]},"915mO7l6Lmz_vWqfnIgwkg":{"text":[" feel that my family gets <em>good</em> care from this vet.  We ended up moving about 15 minutes away, but still"]},"DrFU7tPfdY-95Pm47r72nA":{"text":[" screen that shows a movie (he keeps a <em>good</em> stock of the latest movies.).  Who'd have thought that going"]},"0UBnLhVxAfBI8e36wTupXA":{"text":[" car's transmission went for <em>good</em>, and he decided to fix the Bird (yeah!) so I could drive it (yes, I am"]},"nAJ0HvYhoUjo5wKMewK7iA":{"text":[" their girls and they are <em>good</em>. 6. In fact when it was my birthday month I didn't know I got some discount"]},"ZucxvCvKaw7ViaKU00HYGg":{"text":[" Ice Now! They offer free delivery and their pricing is surprisingly cheap.And <em>good</em> luck getting"]},"M1LxHqmnzohCNyLamfbmow":{"text":["I love Dr. Scott!! He is the funniest doctor I've ever met and he makes me feel <em>good</em> too! Dr. Scott"]},"IjZ33sJrzXqU-0X6U8NwyA":{"text":[" to the waiter to the server. The prices were very <em>good</em> as well. We placed our orders once we decided what we"]},"IESLBzqUCLdSzSqm0eCSxQ":{"text":["love the gyro plate. Rice is so <em>good</em> and I also dig their candy selection :)"]},"1uJFq2r5QfJG_6ExMRCaGw":{"text":["General Manager Scott Petello is a <em>good</em> egg!!! Not to go into detail, but let me assure you if you"]},"riFQ3vxNpP4rWLk_CSri2A":{"text":[" pecans and some fruit in there too which is a yummy bonus!As if the <em>good</em> food wasn't enough to win me"]},"3rPt0LxF7rgmEUrznoH22w":{"text":[" or in one of their many sandwich choices. They have a modest selection of baked <em>goods</em> along with some"]},"cGnKNX3I9rthE0-TH24-qA":{"text":[" <em>good</em>, very fresh - every time.  The 5 stars is for the pizza, but if I were to rate Jason's Deli over"]},"FvEEw1_OsrYdvwLV5Hrliw":{"text":["<em>Good</em> tattoo shop. Clean space, multiple artists to choose from and books of their work"]}}}
"""

LOG_SEARCH_RESPONSE = {
  u'facet_counts': {u'facet_ranges': {u'timestamp': {u'start': u'2014-02-18T12:00:00Z', u'counts': [u'2014-02-26T07:35:00Z', 1, u'2014-02-26T07:40:00Z', 2, u'2014-02-26T07:55:00Z', 20, u'2014-02-26T08:00:00Z', 20, u'2014-02-26T08:10:00Z', 20, u'2014-02-26T08:15:00Z', 21, u'2014-02-26T08:30:00Z', 20, u'2014-02-26T08:45:00Z', 20, u'2014-02-26T09:05:00Z', 20, u'2014-02-26T09:15:00Z', 1, u'2014-02-26T09:30:00Z', 20, u'2014-02-26T09:35:00Z', 20, u'2014-02-26T09:40:00Z', 455, u'2014-02-26T10:10:00Z', 482, u'2014-02-26T10:15:00Z', 21, u'2014-02-26T10:35:00Z', 20, u'2014-02-26T11:05:00Z', 20, u'2014-02-26T11:10:00Z', 20, u'2014-02-26T11:15:00Z', 21, u'2014-02-26T11:20:00Z', 20, u'2014-02-26T11:25:00Z', 20, u'2014-02-26T11:30:00Z', 20, u'2014-02-26T11:40:00Z', 20, u'2014-02-26T11:45:00Z', 20, u'2014-02-26T11:50:00Z', 20, u'2014-02-26T11:55:00Z', 20, u'2014-02-26T12:05:00Z', 5, u'2014-02-26T12:15:00Z', 26, u'2014-02-26T12:20:00Z', 20, u'2014-02-26T12:25:00Z', 20, u'2014-02-26T12:30:00Z', 20, u'2014-02-26T12:35:00Z', 20, u'2014-02-26T13:10:00Z', 21, u'2014-02-26T13:15:00Z', 21, u'2014-02-26T13:20:00Z', 20, u'2014-02-26T13:25:00Z', 20, u'2014-02-26T13:30:00Z', 16, u'2014-02-26T13:35:00Z', 20, u'2014-02-26T13:40:00Z', 20, u'2014-02-26T13:45:00Z', 20, u'2014-02-26T13:55:00Z', 20, u'2014-02-26T14:00:00Z', 20, u'2014-02-26T14:05:00Z', 20, u'2014-02-26T14:10:00Z', 20, u'2014-02-26T14:15:00Z', 1, u'2014-02-26T14:20:00Z', 20, u'2014-02-26T14:25:00Z', 20, u'2014-02-26T14:30:00Z', 20, u'2014-02-26T14:35:00Z', 20, u'2014-02-26T14:45:00Z', 20, u'2014-02-26T14:50:00Z', 16, u'2014-02-26T14:55:00Z', 20, u'2014-02-26T15:00:00Z', 20, u'2014-02-26T15:05:00Z', 20, u'2014-02-26T15:10:00Z', 16, u'2014-02-26T15:15:00Z', 21, u'2014-02-26T15:20:00Z', 16, u'2014-02-26T15:25:00Z', 20, u'2014-02-26T15:30:00Z', 20, u'2014-02-26T15:35:00Z', 20, u'2014-02-26T15:40:00Z', 4, u'2014-02-26T15:45:00Z', 20, u'2014-02-26T15:50:00Z', 16, u'2014-02-26T15:55:00Z', 20, u'2014-02-26T16:05:00Z', 20, u'2014-02-26T16:15:00Z', 21], u'end': u'2014-02-28T12:00:00Z', u'gap': u'+5MINUTES'}}, u'facet_fields': {u'process': [u'kernel', 1740, u'NetworkManager[1089]', 127, u'ntpdate[11671]', 46, u'avahi-daemon[1065]', 35, u'dhclient', 27, u'bluetoothd[1046]', 18, u'ntpd[12759]', 14, u'ntpd[6393]', 13, u'acpid', 12, u'ntpd[12948]', 10, u'dbus[1028]', 8, u'dnsmasq[2265]', 7, u'ntpd_intres[12761]', 6, u'rtkit-daemon[2064]', 6, u'mtp-probe', 4, u'gnome-screensaver-dialog', 3, u'anacron[10740]', 2, u'anacron[11290]', 2, u'anacron[11449]', 2, u'anacron[11760]', 2, u'anacron[12239]', 2, u'anacron[12638]', 2, u'anacron[5916]', 2, u'CRON[10061]', 1, u'CRON[13189]', 1, u'CRON[14491]', 1, u'CRON[16528]', 1, u'CRON[18735]', 1, u'CRON[21616]', 1, u'CRON[25014]', 1, u'CRON[29157]', 1, u'CRON[8406]', 1, u'ntpd[12758]', 1, u'ntpd[12761]', 1, u'ntpd[12947]', 1, u'ntpdate[12895]', 1, u'rsyslogd', 1], u'host': [u'domain5', 444, u'domain4', 426, u'domain1', 425, u'domain2', 423, u'domain3', 386]}, u'facet_dates': {}, u'facet_queries': {}}, u'highlighting': {u'437860700287801386': {}, u'437860700287801387': {}, u'437860700287801384': {}, u'437860700287801385': {}, u'437860700287801391': {}, u'437860700287801404': {}, u'437860700287801393': {}, u'437860700287801392': {}, u'437860700287801397': {}, u'437860700287801388': {}, u'437860700287801389': {}, u'437860700287801395': {}, u'437860700287801396': {}, u'437860700287801394': {}, u'437860700287801390': {}}, u'responseHeader': {"is_demo":True, u'status': 0, u'QTime': 44, u'params': {u'sort': u'timestamp desc', u'q': u'*:*', u'facet.mincount': u'1', u'f.timestamp.facet.range.gap': u'+5MINUTES', u'facet.sort': u'count', u'wt': u'json', u'start': u'0', u'f.timestamp.facet.range.end': u'2014-02-28T12:00:00Z', u'f.timestamp.facet.limit': u'-1', u'doAs': u'romain', u'facet': u'true', u'facet.limit': u'100', u'facet.range': u'timestamp', u'user.name': u'hue', u'hl': u'true', u'hl.fl': u'message', u'rows': u'15', u'facet.field': [u'process', u'host'], u'f.timestamp.facet.range.start': u'2014-02-28T12:00:00Z-10DAYS'}}, u'response': {u'start': 0, u'numFound': 2104, u'docs': [{u'process': u'CRON[29157]', u'timestamp': u'2014-02-26T16:17:01Z', u'host': u'domain2', u'_version_': 1461151540416872470, u'message': u'(root) CMD (   cd / && run-parts --report /etc/cron.hourly)', u'id': u'437860700287801404'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain1', u'_version_': 1461151540416872450, u'message': u'[178117.882479] CPU1: Core temperature above threshold, cpu clock throttled (total events = 316493)', u'id': u'437860700287801384'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain5', u'_version_': 1461151540416872451, u'message': u'[178117.882481] CPU0: Core temperature above threshold, cpu clock throttled (total events = 316493)', u'id': u'437860700287801385'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain4', u'_version_': 1461151540416872452, u'message': u'[178117.882484] CPU0: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801386'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain3', u'_version_': 1461151540416872453, u'message': u'[178117.882486] CPU1: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801387'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain5', u'_version_': 1461151540416872454, u'message': u'[178117.882517] CPU3: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801388'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain4', u'_version_': 1461151540416872455, u'message': u'[178117.882520] CPU2: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801389'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain3', u'_version_': 1461151540416872456, u'message': u'[178117.882523] CPU5: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801390'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain2', u'_version_': 1461151540416872457, u'message': u'[178117.882527] CPU4: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801391'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain3', u'_version_': 1461151540416872458, u'message': u'[178117.882530] CPU6: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801392'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain5', u'_version_': 1461151540416872459, u'message': u'[178117.882533] CPU7: Package temperature above threshold, cpu clock throttled (total events = 489707)', u'id': u'437860700287801393'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain5', u'_version_': 1461151540416872460, u'message': u'[178117.883490] CPU1: Core temperature/speed normal', u'id': u'437860700287801394'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain5', u'_version_': 1461151540416872461, u'message': u'[178117.883492] CPU0: Core temperature/speed normal', u'id': u'437860700287801395'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain3', u'_version_': 1461151540416872462, u'message': u'[178117.883493] CPU0: Package temperature/speed normal', u'id': u'437860700287801396'}, {u'process': u'kernel', u'timestamp': u'2014-02-26T16:15:32Z', u'host': u'domain2', u'_version_': 1461151540416872463, u'message': u'[178117.883494] CPU1: Package temperature/speed normal', u'id': u'437860700287801397'}]}
}